; ------------------------------------------------------------------------------
;
;					X-UDF (User Defined Functions)
;
; ------------------------------------------------------------------------------
;
; AutoIt Version:	3.2.0.1
; Language:			English
; Description:		Functions used in X-Launcher 1.1
; Author:			Gabriele Tittonel <tittoproject@gmail.com>
;
; ------------------------------------------------------------------------------
;
;
;
;===============================================================================
;
; Function Name:	_DirCopy()
; Description:		Copies a directory and all sub-directories and files
; Syntax:			_DirCopy(SourcePath|DestinationPath)
; Requirements:		_FullPath
;
;===============================================================================
Func _DirCopy($string)
	Local $_dircopy, $source, $destination
	$_dircopy = StringSplit($string, '|')
	$source = _FullPath($_dircopy[1], $Root)
	$destination = _FullPath($_dircopy[2], $Root)
	Select
		Case $_dircopy[0] = 3 And $_dircopy[3] = 'o'
			DirCopy($source, $destination, 1)
		Case Else
			DirCopy($source, $destination)
	EndSelect
EndFunc   ;==>_DirCopy

;===============================================================================
;
; Function Name:	_DirCreate()
; Description:		Creates a directory/folder
; Syntax:			_DirCreate(Path1\SubPath1;SubPath2|Path2|Path3\SubPath3)
; Requirements:		_FileInfo & _FullPath
;
;===============================================================================
Func _DirCreate($string)
	Local $_paths, $_dirs, $path1, $path
	$_paths = StringSplit($string, '|')
	For $i = 1 To $_paths[0]
		$_dirs = StringSplit($_paths[$i], ';')
		$path1 = _FullPath($_dirs[1], $Root)
		$path = _FileInfo($path1, 0)
		DirCreate($path1)
		For $d = 2 To $_dirs[0]
			DirCreate($path & '\' & $_dirs[$d])
		Next
	Next
EndFunc   ;==>_DirCreate

;===============================================================================
;
; Function Name:	_DirMove()
; Description:		Renames a file
; Syntax:			_DirMove(Path\File|NewName)
; Requirements:		_FullPath
;
;===============================================================================
Func _DirMove($string)
	Local $_split, $source, $destination
	$_split = StringSplit($string, '|')
	$source = _FullPath ($_split[1])
	$destination = _FullPath ($_split[2])
	If $_split[0] = 3 And $_split[3] = 'o' Then
		DirMove($source, $destination, 9)
	Else
		DirMove($source, $destination, 8)
	EndIf
EndFunc   ;==>_FileDelete

;===============================================================================
;
; Function Name:	_DirName()
; Description:		Returns the name of a directory
; Syntax:			_DirName(X:\Dir\Full\Path)
;
;===============================================================================
Func _DirName($path)
	$_path = StringSplit(_NormalPath($path), '\')
	Return $_path[$_path[0]]
EndFunc   ;==>_DirName

;===============================================================================
;
; Function Name:	_FileCreate()
; Description:      Creates or zero's out the length of the file specified.
; Syntax:			_FileCreate(Path1\File1;File2|File2|Path2\File3)
; Requirements:		_FileInfo , _FullPath & _EmptyFileCreate
;
;===============================================================================
Func _FileCreate($string)
	Local $_paths, $_files, $file1, $path
	$_paths = StringSplit($string, '|')
	For $i = 1 To $_paths[0]
		$_files = StringSplit($_paths[$i], ';')
		$file1 = _FullPath($_files[1], $Root)
		$path = _FileInfo($file1, 0)
		_EmptyFileCreate($file1)
		For $d = 2 To $_files[0]
			_EmptyFileCreate($path & '\' & $_files[$d])
		Next
	Next
EndFunc   ;==>_FileCreate

Func _EmptyFileCreate($filepath)
	Local $write
	$write = FileOpen($filepath, 10)
	If Not $write = -1 Then FileWrite($write, '')
	FileClose($write)
EndFunc   ;==>_EmptyFileCreate

;===============================================================================
;
; Function Name:	_FileCopy()
; Description:		Copy file(s) from source to destination
; Syntax:			_FileCopy(source|destination)
;					source = Path\file1;file2;file3 | destination = Path
;					source = Path\file | destination = Path or File
; Requirements:		_FileInfo & _FullPath
;
;===============================================================================
Func _FileCopy($string)
	Local $_paths, $destination, $_source, $source, $source_dir
	$_paths = StringSplit($string, '|')
	If Not @error Then
		$destination = _FullPath($_paths[2], $Root)
		$_source = StringSplit($_paths[1], ';')
		$source = _FullPath($_source[1], $Root)
		If $_paths[0] = 3 And $_paths[3] = 'o' Then
			$flag = 9
		Else
			$flag = 8
		EndIf
		If $_source[0] <> 1 Then
			For $f = 1 To $_source[0]
				$source_dir = _FileInfo($source, 0)
				FileCopy($source_dir & '\' & $_source[$f], $destination, $flag)
			Next
		Else
			FileCopy($source, $destination, $flag)
		EndIf
	EndIf
EndFunc   ;==>_FileCopy

;===============================================================================
;
; Function Name:	_FileDelete()
; Description:		Deletes files
; Syntax:			_FileDelete(Path1\file1;file2|Path2\file3)
; Requirements:		_FileInfo & _FullPath
;
;===============================================================================
Func _FileDelete($string)
	Local $_paths, $files, $path
	$_paths = StringSplit($string, '|')
	For $i = 1 To $_paths[0]
		$files = StringSplit($_paths[$i], ';')
		$path = _FileInfo($files[1], 0)
		FileDelete(_FullPath($files[1], $Root))
		For $f = 2 To $files[0]
			FileDelete(_FullPath($path & '\' & $files[$f], $Root))
		Next
	Next
EndFunc   ;==>_FileDelete

;===============================================================================
;
; Function Name:	_FileInfo()
; Description:		Returns path, name or extension of a gived file full-path.
; Paremeters:		flag = 0 --> returns "Path"
; 					flag = 1 --> returns "Name+Extension"
; 					flag = 2 --> returns "Name"
; 					flag = 3 --> returns "Extension"
;
;===============================================================================
Func _FileInfo($filepath, $flag)
	Local $path, $file, $dot, $name, $ext
	$filepath = _NormalPath($filepath)
	$path = StringLeft($filepath, StringInStr($filepath, '\', 0, -1) - 1)
	$file = StringTrimLeft($filepath, StringInStr($filepath, '\', 0, -1))
	$dot = StringInStr($file, '.', 0, -1)
	Select
		Case $dot = 0
			$name = $file
			$ext = ''
		Case Else
			$name = StringLeft($file, $dot - 1)
			$ext = StringTrimLeft($file, $dot)
	EndSelect
	Select
		Case $flag = 0
			Return $path
		Case $flag = 1
			Return $file
		Case $flag = 2
			Return $name
		Case $flag = 3
			Return $ext
	EndSelect
EndFunc   ;==>_FileInfo

;===============================================================================
;
; Function Name:	_FileMove()
; Description:		Renames a file
; Syntax:			_FileRename(Path\File|NewName)
; Requirements:		_FullPath
;
;===============================================================================
Func _FileMove($string)
	Local $_split, $source, $destination
	$_split = StringSplit($string, '|')
	$source = _FullPath ($_split[1])
	$destination = _FullPath ($_split[2])
	If $_split[0] = 3 And $_split[3] = 'o' Then
		FileMove($source, $destination, 9)
	Else
		FileMove($source, $destination, 8)
	EndIf
EndFunc   ;==>_FileDelete

;===============================================================================
;
; Function Name:	_FixDriveLetter()
; Description:		If current drive is X:\, then replaces all other drive letters (C:\, D:\, ecc...) in a file.
; Syntax:			_FixDriveLetter(Path\File|skip*)
;					skip=LETTER(S) --> skip letters are not replaced
;					* --> not casesensitive
; Requirements:		_FullPath
;
;===============================================================================
Func _FixDriveLetter($string, $root)
	Local $_string, $file, $casesense, $driveletter, $skip, $_letters[26], $do, $_replace[26], $read, $_read, $lines, $replaces, $write, $_write
	$_string = StringSplit($string, '|')
	$file = _FullPath($_string[1], $root)
	$casesense = 1
	$driveletter = StringLeft($root, 1)
	If $_string[0] > 1 Then
		$skip = $driveletter & $_string[2]
		If StringInStr($skip, '*') Then $casesense = 0
	Else
		$skip = $driveletter
	EndIf
	$_letters[0] = 'A'
	$_letters[1] = 'B'
	$_letters[2] = 'C'
	$_letters[3] = 'D'
	$_letters[4] = 'E'
	$_letters[5] = 'F'
	$_letters[6] = 'G'
	$_letters[7] = 'H'
	$_letters[8] = 'I'
	$_letters[9] = 'J'
	$_letters[10] = 'K'
	$_letters[11] = 'L'
	$_letters[12] = 'M'
	$_letters[13] = 'N'
	$_letters[14] = 'O'
	$_letters[15] = 'P'
	$_letters[16] = 'Q'
	$_letters[17] = 'R'
	$_letters[18] = 'S'
	$_letters[19] = 'T'
	$_letters[20] = 'U'
	$_letters[21] = 'V'
	$_letters[22] = 'W'
	$_letters[23] = 'X'
	$_letters[24] = 'Y'
	$_letters[25] = 'Z'
	$do = 0
	For $s = 0 To 25
		If Not StringInStr($skip, $_letters[$s], $casesense) Then
			$do = $do + 1
			$_replace[$do] = $_letters[$s]
		EndIf
	Next
	If FileExists($file) Then
		$read = FileOpen($file, 0)
		$_read = StringSplit(StringStripCR(FileRead($read, FileGetSize($file))), @LF)
		FileClose($read)
		$lines = $_read[0]
		While 1
			If $_read[$lines] = '' Then
				$lines = $lines - 1
			Else
				ExitLoop
			EndIf
		WEnd
		Local $_write[$lines + 1]
		For $l = 0 To $lines
			For $d = 1 To $do
				If StringInStr($_read[$l], $_replace[$d] & ':\', $casesense) Then
					$_read[$l] = StringReplace($_read[$l], $_replace[$d] & ':\', $driveletter & ':\', '', $casesense)
					$replaces = 1
				EndIf
				If StringInStr($_read[$l], $_replace[$d] & ':/', $casesense) Then
					$_read[$l] = StringReplace($_read[$l], $_replace[$d] & ':/', $driveletter & ':/', '', $casesense)
					$replaces = 1
				EndIf
			Next
			$_write[$l] = $_read[$l]
		Next
		If $replaces = 1 Then
			$write = FileOpen($file, 2)
			For $l = 1 To $lines
				FileWriteLine($write, $_write[$l])
			Next
			FileClose($write)
		EndIf
	EndIf
EndFunc   ;==>_FixDriveLetter

;===============================================================================
;
; Function Name:	_FixUserProfile()
; Description:		Once %USERPROFILE% is defined, renames a directory if its name
;					change when switching OS Lang.
;
;===============================================================================
Func _FixUserProfile($userprofile, $dir, $name)
	Local $cfgfile, $old
	$cfgfile = $userprofile & '\x-launcher.cfg'
	If FileExists($cfgfile) Then
		$old = IniRead($cfgfile, 'UserProfile', $name, '')
		If $dir <> $old Then
			DirMove($userprofile & '\' & $old, $userprofile & '\' & $dir)
			IniWrite($cfgfile, 'UserProfile', $name, $dir)
		EndIf
	Else
		DirCreate($userprofile)
		IniWrite($cfgfile, 'UserProfile', $name, $dir)
	EndIf
EndFunc   ;==>_FixUserProfile

;===============================================================================
;
; Function Name:	_FullPath()
; Description:		Returns the full path from a relative path.
;					\path --> path's relative to current drive's root
;					.\path --> path's relative to root
;					..\path --> path's relative to up level
;					..\..\path --> up two leves, and go on
;
;===============================================================================
Func _FullPath($string, $root = @ScriptDir)
	Local $_slash, $fullpath, $up, $cd, $diff
	If $string = '' Then Return ($string)
	$_slash = StringSplit(_NormalPath($string), '\')
	Select
		Case $_slash[1] = ''
			$fullpath = StringLeft($root, 2) & $string
		Case $_slash[1] = '.'
			$fullpath = $root & StringTrimLeft($string, 1)
		Case $_slash[1] = '..'
			$up = StringSplit($string, '..', 1)
			$cd = StringSplit($root, '\')
			$diff = $cd[0] - $up[0] + 1
			If $diff <= 0 Then Exit
			$fullpath = $cd[1]
			For $i = 2 To $diff
				$fullpath = $fullpath & '\' & $cd[$i]
			Next
			$fullpath = $fullpath & $up[$up[0]]
		Case Else
			$fullpath = $string
	EndSelect
	Return ($fullpath)
EndFunc   ;==>_FullPath

;===============================================================================
;
; Function Name:	_FullPathPlus()
; Description:		Convers a string in a fullpath transformend
; Syntax:			_FullPathPlus(String|Options):
;					~ --> returns the 8.3 short path+name of the string passed
;					%20 --> replaces blanck spaces with '%20'
;					\\ --> replaces '\' with '\\'
;					/ --> replaces '\' with '/'
;					" --> return string enclosed between apexes: "String"
;					= --> the string isn't interpreted as a path
; Requirements:		_FullPath
;
;===============================================================================
Func _FullPathPlus($string)
	Local $_string, $newstring, $options
	$_string = StringSplit($string, '|')
	If @error Then Return (_FullPath($string, $Root))
	$options = $_string[$_string[0]]
	If StringInStr($options, '=') Then
		$newstring = $_string[1]
		For $_s = 2 To $_string[0] - 1
			$newstring = $newstring & '|' & $_string[$_s]
		Next
		Return ($newstring)
	EndIf
	$newstring = _FullPath($_string[1], $Root)
	If StringInStr($options, '~') Then $newstring = FileGetShortName($newstring)
	If StringInStr($options, '%20') Then $newstring = StringReplace($newstring, ' ', '%20')
	If StringInStr($options, '\\') Then $newstring = StringReplace($newstring, '\', '\\')
	If StringInStr($options, '/') Then $newstring = StringReplace($newstring, '\', '/')
	If StringInStr($options, '"') Then $newstring = '"' & $newstring & '"'
	Return ($newstring)
EndFunc   ;==>_FullPathPlus

;===============================================================================
;
; Function Name:	_IniReadPlus()
; Description:		Reads a value from a .ini file, only if it's <> ''
;
;===============================================================================
Func _IniReadPlus($filename, $section, $key, $default)
	Local $var
	$var = IniRead($filename, $section, $key, $default)
	Select
		Case $var = ''
			Return $default
		Case Else
			Return $var
	EndSelect
EndFunc

;===============================================================================
;
; Function Name:	_MozPrefs()
; Description:		Writes to Mozilla's configuration files
; Syntax:			_MozPrefs(File, Preference, Value, User)
;					Mode = Global --> global preference
;					Mode = User --> user preference
; Requirements:		_FileInfo, _FullPath & _RegKeyLoad
;
;===============================================================================
Func _MozPrefs($file, $pref, $value, $mode)
	Local $begin, $end, $replaces = 0, $exists = 0
	Select
		Case $mode = 'Global'
			$begin = 'pref("'
		Case $mode = 'User'
			$begin = 'user_pref("'
	EndSelect
	If FileExists($file) Then
		Local $read, $_read, $lines
		$read = FileOpen($file, 0)
		$_read = StringSplit(StringStripCR(FileRead($read, FileGetSize($file))), @LF)
		FileClose($read)
		$lines = $_read[0]
		While 1
			If $_read[$lines] = '' Then
				$lines = $lines - 1
			Else
				ExitLoop
			EndIf
		WEnd
		Local $_write[$lines + 1]
		For $l = 0 To $lines
			If StringInStr($_read[$l], $pref) <> 0 Then
				$_write[$l] = $begin & $pref & '", ' & $value & ');'
				If $_write[$l] <> $_read[$l] Then $replaces = 1
				$exists = 1
			Else
				$_write[$l] = $_read[$l]
			EndIf
		Next
		If $replaces = 1 Then
			$write = FileOpen($file, 2)
			For $l = 1 To $lines
				FileWriteLine($write, $_write[$l])
			Next
			FileClose($write)
		EndIf
		If $exists = 0 Then
			Local $wfile
			$wfile = FileOpen($file, 1)
			FileWriteLine($wfile, $begin & $pref & '", ' & $value & ');')
			FileClose($wfile)
		EndIf
	EndIf
	If Not FileExists($file) Then
		Local $newfile
		$newfile = FileOpen($file, 9)
		FileWriteLine($newfile, $begin & $pref & '", ' & $value & ');')
		FileClose($newfile)
	EndIf
EndFunc   ;==>_MozPrefs

;===============================================================================
;
; Function Name:	_NormalPath()
; Description:		Returns a standard path string with only singles backslash
;
;===============================================================================
Func _NormalPath($path)
	Local $_normalpath, $normalpath
	$_normalpath = StringSplit(StringReplace($path, '/', '\'), '\')
	$normalpath = $_normalpath[1]
	For $n = 2 To $_normalpath[0]
		If $_normalpath[$n] <> '' Then $normalpath = $normalpath & '\' & $_normalpath[$n]
	Next
	Return $normalpath
EndFunc

;===============================================================================
;
; Function Name:	_RegKeyBackup()
; Description:		Backups a reg key in a *.reg file
;
;===============================================================================
Func _RegKeyBackup($key, $file)
	Local $regedit
	$regedit = @WindowsDir & '\regedit.exe'
	Select
		Case @OSTYPE = 'WIN32_NT'
			RunWait($regedit & ' /s /a /e "' & $file & '" "' & $key & '"', @WindowsDir)
		Case Else
			RunWait($regedit & ' /s /e "' & $file & '" "' & $key & '"', @WindowsDir)
	EndSelect
EndFunc   ;==>_RegKeyBackup

;===============================================================================
;
; Function Name:	_RegKeyGet()
; Description:		Returns the main key contained in a *.reg file
;
;===============================================================================
Func _RegKeyGet($file)
	Local $_keys
	$_keys = IniReadSectionNames($file)
	If Not @error Then Return ($_keys[1])
EndFunc   ;==>_RegKeyGet

;===============================================================================
;
; Function Name:	_RegFileInstall()
; Description:		Installs *.reg file(s). If the key already exists, it will
;					backuped in folder to be restored with _RegFileRestore()
; Syntax:			_RegFileInstall(Path\File1.reg;File2|Path2\File3.reg, Backup dir)
; Requirements:		_FileInfo, _FullPath & _RegKeyLoad
;
;===============================================================================
Func _RegFileInstall($string, $backupdir)
	Local $_files, $_regs, $reg1, $path, $regfile
	DirCreate($backupdir)
	$_files = StringSplit($string, '|')
	For $i = 1 To $_files[0]
		$_regs = StringSplit($_files[$i], ';')
		$reg1 = _FullPath($_regs[1], $Root)
		$path = _FileInfo($reg1, 0)
		_RegKeyLoad($reg1, $backupdir & '\backup-' & $i & 1 & '.reg')
		For $r = 2 To $_regs[0]
			$regfile = $path & '\' & $_regs[$r]
			_RegKeyLoad($regfile, $backupdir & '\backup-' & $i & $r & '.reg')
		Next
	Next
EndFunc   ;==>_RegFileInstall

;===============================================================================
;
; Function Name:	_RegKeyLoad()
; Description:		Installs a single *.reg file. If the key already exists, it
;					will backuped in folder to be restored with _RegFileRestore()
;
;===============================================================================
Func _RegKeyLoad($regfile, $backupfile)
	Local $regedit, $_keys
	$regedit = @WindowsDir & '\regedit.exe'
	$_keys = IniReadSectionNames($regfile)
	If Not @error Then
		RunWait($regedit & ' /s /e "' & $backupfile & '" "' & $_keys[1] & '"', @WindowsDir)
		RegDelete($_keys[1])
		RunWait($regedit & ' /s /i "' & $regfile & '"', @WindowsDir)
	EndIf
EndFunc   ;==>_RegKeyLoAdlibDisable()

;===============================================================================
;
; Function Name:	_RegFileRestore()
; Description:		Restores the original reg keys previously backuped from _RegFileInstall()
; Requirements:		_FullPath, _FileInfo, _RegKeyGet, _RegKeyBackup
;
;===============================================================================
Func _RegFileRestore($string, $backupdir)
	Local $_files, $_f, $_regs, $reg1, $key1, $path, $reg, $key, $_backups, $backup, $regedit
	$_files = StringSplit($string, '|')
	If $_files[$_files[0]] = '*' Then
		$_f = $_files[0] - 1
	Else
		$_f = $_files[0]
	EndIf
	For $i = 1 To $_f
		$_regs = StringSplit($_files[$i], ';')
		$reg1 = _FullPath($_regs[1], $Root)
		$key1 = _RegKeyGet($reg1)
		_RegKeyBackup($key1, $reg1)
		$path = _FileInfo($reg1, 0)
		If $_files[$_files[0]] <> '*' Then RegDelete($key1)
		For $r = 2 To $_regs[0]
			$reg = $path & '\' & $_regs[$r]
			$key = _RegKeyGet($reg)
			_RegKeyBackup($key, $reg)
			If $_files[$_files[0]] <> '*' Then RegDelete($key)
		Next
	Next
	If $_files[$_files[0]] <> '*' Then
		$_backups = FileFindFirstFile($backupdir & '\*.reg')
		If $_backups <> - 1 Then
			$regedit = @WindowsDir & '\regedit.exe'
			While 1
				$backup = FileFindNextFile($_backups)
				If @error Then ExitLoop
				RunWait($regedit & ' /s /i "' & $backupdir & '\' & $backup & '"', @WindowsDir)
			WEnd
		EndIf
		FileClose($_backups)
	EndIf
	DirRemove($backupdir, 1)
EndFunc   ;==>_RegFileRestore

;===============================================================================
;
; Function Name:	_RunWait()
; Description:		Runs an external program
; Syntax:			_RunWait(Path\File|Parameters)
; Requirements:		_FileInfo & _FullPath
;
;===============================================================================
Func _RunWait($string, $root)
	Local $_split, $filepath, $ext, $parameters, $ste
	$_split = StringSplit($string, '|')
	$filepath = _FullPath($_split[1], $root)
	$ext = _FileInfo($filepath, 3)
	If $_split[0] = 2 And $_split[2] <> '' Then
		Select
			Case StringLeft($_split[2], 1) = '-' Or StringLeft($_split[2], 1) = '/'
				$parameters = ' ' & $_split[2]
			Case Else
				$parameters = ' "' & $_split[2] & '"'
		EndSelect
	Else
		$parameters = ''
	EndIf
	$ste = '"' & $filepath & '"' & $parameters
	Select
		Case $ext = 'exe' Or $ext = 'bat' Or $ext = 'com' Or $ext = 'pif'
			RunWait($ste)
		Case Else
			RunWait(@ComSpec & ' /c ' & $ste, '', @SW_HIDE)
	EndSelect
EndFunc   ;==>_RunFile

;===============================================================================
;
; Function Name:	_SetPath()
; Description:		Sets the environmental variable %PATH%
; Syntax:			_SetPath(%PATH%;Path1;Path2)
; Requirements:		_FullPath
;
;===============================================================================
Func _SetPath($string, $log, $logfile)
	Local $_paths, $path
	If $string <> '' Then
		$_paths = StringSplit($string, ';')
		$path = _FullPath($_paths[1], $Root)
		For $i = 2 To $_paths[0]
			$path = $path & ';' & _FullPath($_paths[$i], $Root)
		Next
		EnvSet('PATH', $path)
		If $log = 'true' Then IniWrite($logfile, 'Environment', 'PATH', $path)
	EndIf
EndFunc   ;==>_SetPath

;===============================================================================
;
; Function Name:	_SetEnv()
; Description:		Sets an environmental variable
; Requirements:		_FullPathPlus
;
;===============================================================================
Func _SetEnv($var, $value, $log, $logfile)
	$value = _FullPathPlus ($value)
	EnvSet($var, $value)
	If $log = 'true' Then IniWrite($logfile, 'Environment', $var, $value)
EndFunc
	
;===============================================================================
;
; Function Name:	_SplashScreen()
; Description:		Shows a nice splash screen while starting the file to run
; Requirements:		_FullPath
;
;===============================================================================
Func _SplashScreen($title, $img, $time, $tempdir, $rootdir)
	Local $opt = '', $imgDir, $imgFile, $oShellApp, $oDir, $oFile, $_width, $_height, $width, $height
	Select
		Case $img <> '' And FileExists($img)
			$img = _FullPath($img, $rootdir)
			$imgDir = _FileInfo ($img, 0)
			$imgFile = _FileInfo ($img, 1)
		Case Else
			$img = $tempdir & '\x-splash.jpg'
			FileInstall('x-splash.jpg', $img, 1)
			$imgDir = $tempdir
			$imgFile = 'x-splash.jpg'
	EndSelect
	$oShellApp = ObjCreate('shell.application')
	$oDir = $oShellApp.NameSpace ($imgDir)
	$oFile = $oDir.Parsename ($imgFile)
	$_width = StringSplit($oDir.GetDetailsOf ($oFile, 27), ' ')
	$_height = StringSplit($oDir.GetDetailsOf ($oFile, 28), ' ')
	$width = $_width[1]
	$height = $_height[1]
	If $title = '' Then $opt = 1
	SplashImageOn($title, $img, $width, $height, -1, -1, $opt)
	Sleep($time)
	SplashOff()
	FileDelete($tempdir & '\x-splash.jpg')
EndFunc   ;==>_SplashScreen

;===============================================================================
;
; Function Name:	_StringReplace()
; Description:		Replaces a string between two others one in a file
;
;===============================================================================
Func _StringReplace($file, $begin, $end, $replace)
	Local $read, $_read, $lines, $_begin, $_end, $replaces = 0, $write, $_write
	If FileExists($file) Then
		$read = FileOpen($file, 0)
		$_read = StringSplit(StringStripCR(FileRead($read, FileGetSize($file))), @LF)
		FileClose($read)
		$lines = $_read[0]
		While 1
			If $_read[$lines] = '' Then
				$lines = $lines - 1
			Else
				ExitLoop
			EndIf
		WEnd
		Local $_write[$lines + 1]
		For $l = 0 To $lines
			If StringInStr($_read[$l], $begin) <> 0 And StringInStr($_read[$l], $end) <> 0 Then
				$_begin = StringSplit($_read[$l], $begin, 1)
				$_end = StringSplit($_read[$l], $end, 1)
				$_write[$l] = $_begin[1] & $begin & $replace & $end & $_end[2]
				If $_write[$l] <> $_read[$l] Then $replaces = 1
			Else
				$_write[$l] = $_read[$l]
			EndIf
		Next
		If $replaces = 1 Then
			$write = FileOpen($file, 2)
			For $l = 1 To $lines
				FileWriteLine($write, $_write[$l])
			Next
			FileClose($write)
		EndIf
	EndIf
EndFunc   ;==>_StringReplace

;===============================================================================
;
; Function Name:	_WriteToFile()
; Description:		Writes text to a specific line in a file
; Syntax:			_WriteLine(File, LineX or EOF, String)
;
;===============================================================================
Func _WriteToFile($file, $line, $content)
	Local $ftw, $lineNum, $read, $_read, $lines, $write
	Select
		Case $line = 'EOF'
			$ftw = FileOpen($file, 9)
			FileWriteLine($ftw, $content)
			FileClose($ftw)
		Case StringInStr($line, 'Line')
			$lineNum = StringTrimLeft($line, 4)
			If StringIsInt($lineNum) Then
				If FileExists($file) Then
					$read = FileOpen($file, 0)
					$_read = StringSplit(StringStripCR(FileRead($read, FileGetSize($file))), @LF)
					FileClose($read)
					$lines = $_read[0]
					While 1
						If $_read[$lines] = '' Then
							$lines = $lines - 1
						Else
							ExitLoop
						EndIf
					WEnd
					$write = FileOpen($file, 2)
					If $lineNum <= $lines Then $_read[$lineNum] = $content
					For $l = 1 To $lines
						FileWriteLine($write, $_read[$l])
					Next
					If $lineNum > $lines Then
						For $l = $lines + 1 To $lineNum - 1
							FileWriteLine($write, '')
						Next
						FileWriteLine($write, $content)
					EndIf
					FileClose($write)
				Else
					$ftw = FileOpen($file, 9)
					For $l = 1 To $lineNum - 1
						FileWriteLine($ftw, '')
					Next
					FileWriteLine($ftw, $content)
					FileClose($ftw)
				EndIf
			EndIf
	EndSelect
EndFunc   ;==>_WriteToFile

;===============================================================================
;
; Function Name:	_WriteToPref()
; Description:		Writes to Mozilla's configuration files
; Syntax:			_WriteToPrefs(File, Format, Pref, Value)
; Requirements:		_FileInfo, _FullPath & _RegKeyLoad
;
;===============================================================================
Func _WriteToPref($file, $begin, $mid, $end, $pref, $value)
	Local $replaces = 0, $exists = 0
	If FileExists($file) Then
		Local $read, $_read, $lines
		$read = FileOpen($file, 0)
		$_read = StringSplit(StringStripCR(FileRead($read, FileGetSize($file))), @LF)
		FileClose($read)
		$lines = $_read[0]
		While 1
			If $_read[$lines] = '' Then
				$lines = $lines - 1
			Else
				ExitLoop
			EndIf
		WEnd
		Local $_write[$lines + 1]
		For $l = 0 To $lines
			If StringInStr($_read[$l], $begin & $pref & $mid) <> 0 Then
				$_write[$l] = $begin & $pref & $mid & $value & $end
				If $_write[$l] <> $_read[$l] Then $replaces = 1
				$exists = 1
			Else
				$_write[$l] = $_read[$l]
			EndIf
		Next
		If $replaces = 1 Then
			$write = FileOpen($file, 2)
			For $l = 1 To $lines
				FileWriteLine($write, $_write[$l])
			Next
			FileClose($write)
		EndIf
		If $exists = 0 Then
			Local $wfile
			$wfile = FileOpen($file, 1)
			FileWriteLine($wfile, $begin & $pref & $mid & $value & $end)
			FileClose($wfile)
		EndIf
	Else
		Local $newfile
		$newfile = FileOpen($file, 9)
		FileWriteLine($newfile, $begin & $pref & $mid & $value & $end)
		FileClose($newfile)
	EndIf
EndFunc   ;==>_WriteToPref

;===============================================================================
;
; Function Name:	_GetSID()
; Description:		Restituisce il Security IDentifier
;
;===============================================================================
Func _GetSID()
	Local $wbemFlagReturnImmediately = 0x10, $wbemFlagForwardOnly = 0x20, $colItems = "", $Output="", $user = "", $sid = ""
	$objWMIService = ObjGet("winmgmts:\\" & @ComputerName & "\root\CIMV2")
	$colItems = $objWMIService.ExecQuery("SELECT * FROM Win32_AccountSID", "WQL", $wbemFlagReturnImmediately + $wbemFlagForwardOnly)
	If IsObj($colItems) then
		For $objItem In $colItems
			$user = StringReplace(StringMid($objItem.Element, StringInStr($objItem.Element, "Name=")+5), Chr(34), "")
			If ($user = @UserName) Then
				$sid = StringReplace(StringMid($objItem.Setting, StringInStr($objItem.Setting, "SID=")+4), Chr(34), "")
				ExitLoop
			EndIf
		Next
	EndIf
	return $sid
EndFunc

;===============================================================================
;
; Function Name:	_isOnlySID()
; Description:		Controlla se  cambiato solo il SID
;
;===============================================================================
Func _isOnlySID($file, $mainkey)
	Local $old, $new, $i
	$old = _RegKeyGet($file)
	$new = $mainkey
	If (StringLeft($old, 10) = "HKEY_USERS" And StringLeft($new, 10) = "HKEY_USERS") Then
		$old = StringTrimLeft($old, 11)
		$new = StringTrimLeft($new, 11)
		For $i = 1 To StringLen($old)
			If (StringLeft($old, 1) <> "\") Then
				$old = StringTrimLeft($old, 1)
			EndIf
		Next
		For $i = 1 To StringLen($new)
			If (StringLeft($new, 1) <> "\") Then
				$new = StringTrimLeft($new, 1)
			EndIf
		Next
		If ($old = $new) Then
			Return True
		EndIf
	EndIf
	Return False
EndFunc

;===============================================================================
;
; Function Name:	_ChangeSID()
; Description:		Cambia il vecchio SID con il nuovo SID
;
;===============================================================================
func _ChangeSID($file)
	Local $temp, $old, $new, $read, $_read, $write, $i
	$temp = StringTrimLeft(_RegKeyGet($file), 11)
	For $i = 1 To StringLen($temp)
		$car = StringLeft($temp, 1)
		$temp = StringTrimLeft($temp, 1)
		If ($car <> "\") Then
			$old = $old & $car
		Else
			ExitLoop
		EndIf
	Next
	$new = _GetSID()
	$read = FileOpen($file, 0)
	$_read = StringSplit(StringStripCR(FileRead($read, FileGetSize($file))), @LF)
	FileClose($read)
	$write = FileOpen($file & ".tmp", 2)
	For $i = 1 To $_read[0]-1
		FileWriteLine($write, StringReplace($_read[$i], $old, $new))
	Next
	FileClose($write)
	$read = FileOpen($file & ".tmp", 0)
	$_read = StringSplit(StringStripCR(FileRead($read, FileGetSize($file & ".tmp"))), @LF)
	FileClose($read)
	FileDelete($file & ".tmp")
	$write = FileOpen($file, 2)
	For $i = 1 To $_read[0]-1
		FileWriteLine($write, $_read[$i])
	Next
	FileClose($write)
EndFunc

;===============================================================================
;
; Function Name:	_WriteToReg()
; Description:		Writes text to a specific line in a file
; Syntax:			_WriteLine(File, LineX or EOF, String)
;
;===============================================================================
Func _WriteToReg($file, $mainkey, $subkey, $name, $value)
	Local $write, $clear, $key

	;	Sostituisce tutte le occorrenze della parola chiave "@SID@" con il vero SID
	;	corrispondente all'account utente
	;
	;	ESEMPIO:
	;	[WriteToReg=$Home$\HKU.reg]
	;	MainKey=HKEY_USERS\@SID@\Software\PepiMK Software
	;
	;	MainKey diventa ad esempio:
	;	HKEY_USERS\S-1-5-21-1801674531-789336058-682003330-1005\Software\PepiMK Software

	$mainkey = StringReplace($mainkey, "@SID@", _GetSID())
	
	If Not FileExists($file) Then
		$write = FileOpen($file, 10)
		FileWriteLine($write, 'REGEDIT4')
		FileWriteLine($write, '[' & $mainkey & ']')
		FileClose($write)
	ElseIf FileReadLine($file, 1) <> 'REGEDIT4' Or _RegKeyGet($file) <> $mainkey Then

		;	Se il file .reg  gi esistente, e la mainkey contenuta al suo interno  diversa da quella contenuta nel file .ini,
		;	allora confronta le due mainkey per vedere se  cambiato solo il SID.
		;	In caso affermativo sostituisce nel file .reg tutte le occorrenze del vecchio SID con il nuovo SID,
		;	altrimenti cancella il contenuto del file .reg e lo aggiorna con la nuova mainkey.
		;
		;	ESEMPIO:
		;	REGEDIT4
		;
		;	[HKEY_USERS\VECCHIO_SID\Software\PepiMK Software]
		;
		;	[HKEY_USERS\VECCHIO_SID\Software\PepiMK Software\SpybotSnD]
		;	"Version"="1.4"
		;	"Path"="E:\\Utility\\Spybot - Search & Destroy\\_source\\X-Launcher_src\\Bin\\"
		;	"WizardRun"=dword:00000001
		;	"ShortcutbarWidth"=dword:00000081
		;	"ResultsDir"="C:\\Documents and Settings\\All Users\\Dati applicazioni\\Spybot - Search & Destroy"
		;
		;	[HKEY_USERS\VECCHIO_SID\Software\PepiMK Software\SpybotSnD\Download directories]
		;
		;
		;	Se la mainkey del file x-launcher.ini 
		;	HKEY_USERS\NUOVO_SID\Software\PepiMK Software
		;	
		;	allora il file .reg diventa:
		;	REGEDIT4
		;
		;	[HKEY_USERS\NUOVO_SID\Software\PepiMK Software]
		;
		;	[HKEY_USERS\NUOVO_SID\Software\PepiMK Software\SpybotSnD]
		;	"Version"="1.4"
		;	"Path"="E:\\Utility\\Spybot - Search & Destroy\\_source\\X-Launcher_src\\Bin\\"
		;	"WizardRun"=dword:00000001
		;	"ShortcutbarWidth"=dword:00000081
		;	"ResultsDir"="C:\\Documents and Settings\\All Users\\Dati applicazioni\\Spybot - Search & Destroy"
		;
		;	[HKEY_USERS\NUOVO_SID\Software\PepiMK Software\SpybotSnD\Download directories]
		;
		;
		;	Questi controlli sono utili nel caso in cui si utilizza il programma su un altro computer o su un altro
		;	account utente.
		
		If _isOnlySID($file, $mainkey) Then
			_ChangeSID($file)
		Else
			$clear = FileOpen($file, 2)
			FileWriteLine($clear, 'REGEDIT4')
			FileWriteLine($clear, '[' & $mainkey & ']')
			FileClose($clear)
		EndIf
	EndIf
	Select
		Case $subkey = '' And $name = '' And $value = ''
		Case Else
			If $subkey = '' Then
				$key = $mainkey
			Else
				$key = $mainkey & '\' & $subkey
			EndIf
			If $value <> '' Then $value = _FullPathPlus($value)
			IniWrite($file, $key, $name, $value)
	EndSelect
EndFunc   ;==>_WriteToReg
